/*
 * @Author: zhoufei
 * @Date: 2022-04-22 14:58:25
 * @LastEditors: zhoufei
 * @LastEditTime: 2022-04-24 18:48:25
 * @FilePath: /utils/queue/rocketmq.go
 * @Description:rocketmq 工具类
 *
 * Copyright (c) 2022 by zhoufei, All Rights Reserved.
 */
package queue

import (
	"context"
	"errors"
	"fmt"

	"github.com/apache/rocketmq-client-go/v2"
	"github.com/apache/rocketmq-client-go/v2/consumer"
	"github.com/apache/rocketmq-client-go/v2/primitive"
	"github.com/apache/rocketmq-client-go/v2/producer"
)

type RocketMq struct {
	endPoints []string
	producer  rocketmq.Producer
	consumer  rocketmq.PushConsumer
}

type rocketOptions struct {
	AccessKey string
	SecretKey string
}

type RocketOption func(*rocketOptions)

func WithRocketAcl(username string, password string) RocketOption {
	return func(o *rocketOptions) {
		if username == "" || password == "" {
			return
		}
		o.AccessKey = username
		o.SecretKey = password
	}
}

/**
 * @description: 发送消息
 * @param {string} topic
 * @param {[]byte} msg
 * @return {*}
 */
func (r *RocketMq) SendMsg(topic string, msg []byte) (qMsg *QueueMsg, err error) {
	if r.producer == nil {
		return nil, errors.New("no producer available")
	}

	resp, err := r.producer.SendSync(context.Background(), primitive.NewMessage(topic, msg))

	if err != nil {
		return
	}
	if resp.Status != primitive.SendOK {
		return nil, fmt.Errorf("error sending message,status:%v", resp.Status)
	}
	qmsg := &QueueMsg{
		Body:  []byte(msg),
		Topic: topic,
		Info:  map[string]interface{}{"msgId": resp.MsgID},
	}
	return qmsg, err
}

/**
 * @description: 发送字符串消息
 * @param {string} topic
 * @param {string} msg
 * @return {*}
 */
func (r *RocketMq) SendStringMsg(topic string, msg string) (qMsg *QueueMsg, err error) {
	return r.SendMsg(topic, []byte(msg))
}

/**
 * @description: 注册消费者回调
 * @param {*QueueMsg} qMsg
 * @return {*}
 */
func (r *RocketMq) RegisterConsumer(topic string, recevieFunc func(qMsg *QueueMsg)) (err error) {
	if r.consumer == nil {
		return errors.New("no consumer available")
	}

	err = r.consumer.Subscribe(topic, consumer.MessageSelector{}, func(_ context.Context, msgs ...*primitive.MessageExt) (consumer.ConsumeResult, error) {
		for _, msg := range msgs {
			go recevieFunc(&QueueMsg{
				Body:  msg.Body,
				Topic: topic,
				Info:  map[string]interface{}{"msgId": msg.MsgId},
			})
		}
		return consumer.ConsumeSuccess, nil
	})

	if err != nil {
		return err
	}

	err = r.consumer.Start()
	if err != nil {
		r.consumer.Unsubscribe(topic)
		return err
	}
	return nil
}

func (r *RocketMq) ShutDown() error {
	var err error
	if r.producer != nil {
		err = r.producer.Shutdown()
	}
	if r.consumer != nil {
		err = r.consumer.Shutdown()
	}
	return err
}

/**
 * @description: 创建rocketmq生产者
 * @param {[]string} endPoints nameserver
 * @param {string} groupName
 * @param {int} retry
 * @return {*}
 */
func NewRocketMqProducer(endPoints []string, groupName string, retry int, options ...RocketOption) (p Producer, err error) {

	if retry <= 0 {
		retry = 0
	}

	opts := []producer.Option{producer.WithGroupName(groupName),
		producer.WithNsResolver(primitive.NewPassthroughResolver(endPoints)),
		producer.WithRetry(retry)}

	ropts := &rocketOptions{}
	for _, v := range options {
		v(ropts)
	}

	if ropts.AccessKey != "" && ropts.SecretKey != "" {
		opts = append(opts, producer.WithCredentials(primitive.Credentials{
			AccessKey: ropts.AccessKey,
			SecretKey: ropts.SecretKey,
		}))
	}

	client, err := rocketmq.NewProducer(opts...)
	if err != nil {
		return nil, err
	}

	r := &RocketMq{
		endPoints: endPoints,
		producer:  client,
	}

	err = r.producer.Start()
	if err != nil {
		return nil, err
	}
	return r, nil
}

/**
 * @description: 创建rocketmq消费者
 * @param {[]string} endPoints nameserver
 * @param {string} groupName
 * @return {*}
 */
func NewRocketMqConsumer(endPoints []string, groupName string, options ...RocketOption) (c Consumer, err error) {
	opts := []consumer.Option{
		consumer.WithNsResolver(primitive.NewPassthroughResolver(endPoints)),
		consumer.WithGroupName(groupName),
		consumer.WithConsumerModel(consumer.Clustering)}

	ropts := &rocketOptions{}
	for _, v := range options {
		v(ropts)
	}

	if ropts.AccessKey != "" && ropts.SecretKey != "" {
		opts = append(opts, consumer.WithCredentials(primitive.Credentials{
			AccessKey: ropts.AccessKey,
			SecretKey: ropts.SecretKey,
		}))
	}

	client, err := rocketmq.NewPushConsumer(opts...)

	if err != nil {
		return nil, err
	}

	r := &RocketMq{
		endPoints: endPoints,
		consumer:  client,
	}

	return r, nil
}
